"""Some simple yaml file reader"""

from __future__ import print_function

import ast
import json
import os
import stat
import sys
from collections import OrderedDict
from pprint import pprint
from string import Template

import simplejson
import yaml

from cloudmesh_client.common.Error import Error
from cloudmesh_client.common.util import backup_name, path_expand
from cloudmesh_client.locations import config_file
from cloudmesh_client.logger import LOGGER
from cloudmesh_client.shell.console import Console

log = LOGGER(__file__)
package_dir = os.path.dirname(os.path.abspath(__file__))
attribute_indent = 4


def check_file_for_tabs(filename, verbose=True):
    """identifies if the file contains tabs and returns True if it
    does. It also prints the location of the lines and columns. If
    verbose is set to False, the location is not printed.

    :param filename: the filename
    :rtype: True if there are tabs in the file
    """
    file_contains_tabs = False
    with open(filename) as f:
        lines = f.read().split("\n")

    line_no = 1
    for line in lines:
        if "\t" in line:
            file_contains_tabs = True
            location = [
                i for i in range(len(line)) if line.startswith('\t', i)]
            if verbose:
                Console.error("Tab found in line {} and column(s) {}".format(line_no, str(location).replace("[","").replace("]", "")), traceflag=False)
        line_no += 1
    return file_contains_tabs


# http://stackoverflow.com/questions/5121931/in-python-how-can-you-load-yaml-mappings-as-ordereddicts

def ordered_load(stream, Loader=yaml.Loader, object_pairs_hook=OrderedDict):
    """
    Loads an ordered dict into a yaml while preserving the order

    :param stream: the name of the stream
    :param Loader: the yam loader (such as yaml.SafeLoader)
    :param object_pairs_hook: the ordered dict
    """

    class OrderedLoader(Loader):
        pass

    def construct_mapping(loader, node):
        loader.flatten_mapping(node)
        return object_pairs_hook(loader.construct_pairs(node))

    OrderedLoader.add_constructor(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
        construct_mapping)
    return yaml.load(stream, OrderedLoader)


# usage example:
# ordered_load(stream, yaml.SafeLoader)


def ordered_dump(data, stream=None, Dumper=yaml.Dumper, **kwds):
    """
    writes the dict into an ordered yaml.

    :param data: The ordered dict
    :param stream: the stream
    :param Dumper: the dumper such as yaml.SafeDumper
    """

    class OrderedDumper(Dumper):
        pass

    def _dict_representer(dumper, data):
        return dumper.represent_mapping(
            yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
            data.items())

    OrderedDumper.add_representer(OrderedDict, _dict_representer)
    return yaml.dump(data, stream, OrderedDumper, **kwds)


# usage:
# ordered_dump(data, Dumper=yaml.SafeDumper)


def read_yaml_config(filename, check=True, osreplace=True, exit=True):
    """
    reads in a yaml file from the specified filename. If check is set to true
    the code will fail if the file does not exist. However if it is set to
    false and the file does not exist, None is returned.

    :param filename: the file name
    :param check: if True fails if the file does not exist,
                  if False and the file does not exist return will be None
    """
    location = filename
    if location is not None:
        location = path_expand(location)

    if not os.path.exists(location) and not check:
        return None

    if check and os.path.exists(location):

        # test for tab in yaml file
        if check_file_for_tabs(location):
            log.error("The file {0} contains tabs. yaml "
                      "Files are not allowed to contain tabs".format(location))
            sys.exit()
        result = None
        try:

            if osreplace:
                result = open(location, 'r').read()
                t = Template(result)
                result = t.substitute(os.environ)

                # data = yaml.safe_load(result)
                data = ordered_load(result, yaml.SafeLoader)
            else:
                f = open(location, "r")

                # data = yaml.safe_load(f)

                data = ordered_load(result, yaml.SafeLoader)
                f.close()

            return data
        except Exception as e:
            log.error(
                "The file {0} fails with a yaml read error".format(filename))
            Error.traceback(e)
            sys.exit()

    else:
        log.error("The file {0} does not exist.".format(filename))
        if exit:
            sys.exit()

    return None


class OrderedJsonEncoder(simplejson.JSONEncoder):
    indent = attribute_indent

    def encode(self, o, depth=0):
        if isinstance(o, OrderedDict):
            return "{" + ",\n ".join([self.encode(k) + ":" +
                                      self.encode(v, depth + 1)
                                      for (k, v) in o.items()]) + "}\n"
        else:
            return simplejson.JSONEncoder.encode(self, o)


def custom_print(data_structure, indent):
    for key, value in data_structure.items():
        print("\n%s%s:" % (' ' * attribute_indent * indent, str(key)), end=' ')
        if isinstance(value, OrderedDict):
            custom_print(value, indent + 1)
        elif isinstance(value, dict):
            custom_print(value, indent + 1)
        else:
            print("%s" % (str(value)), end=' ')


class BaseConfigDict(OrderedDict):
    """
    A class to obtain an OrderedDict from a yaml file.
    """

    def _set_filename(self, filename):
        """
        Sets the filename to be used.

        :param filename: the filename
        """
        self['filename'] = filename
        self['location'] = path_expand(self["filename"])

    def __init__(self, *args, **kwargs):
        """
        The initalization method
        """
        OrderedDict.__init__(self, *args, **kwargs)

        if 'filename' in kwargs:
            self._set_filename(kwargs['filename'])
        else:
            log.error("filename not specified")
            # sys.exit()

        if os.path.isfile(self['location']):
            self.load(self['location'])

        # print ("ATTRIBUTE", attribute)
        for attribute in ['prefix']:
            if attribute in kwargs:
                self[attribute] = kwargs[attribute]
            else:
                self[attribute] = None

        self._update_meta()

    def _update_meta(self):
        """
        internal function to define the metadata regarding filename, location,
        and prefix.
        """
        for v in ["filename", "location", "prefix"]:
            if "meta" not in self:
                self["meta"] = {}
            self["meta"][v] = self[v]
            del self[v]

    def read(self, filename):
        """
        Loads the information in the yaml file. It is the same as load and is
        used for compatibility reasons.

        :param filename: the name of the yaml file
        """
        self.load(filename)

    def load(self, filename):
        """
        Loads the yaml file with the given filename.

        :param filename: the name of the yaml file
        """

        self._set_filename(filename)

        if os.path.isfile(self['location']):
            # d = OrderedDict(read_yaml_config(self['location'], check=True))
            d = read_yaml_config(self['location'], check=True)
            with open(self['location']) as myfile:
                document = myfile.read()
            x = yaml.load(document)
            try:
                self.update(d)
            except:
                print("ERROR: can not find", self["location"])
                sys.exit()
        else:
            print("Error while reading and updating the configuration file {:}".format(filename))

    def make_a_copy(self, location=None):
        """
        Creates a backup of the file specified in the location. The backup
        filename  appends a .bak.NO where number is a number that is not yet
        used in the backup directory.

        TODO: This function should be moved to another file maybe XShell

        :param location: the location of the file to be backed up
        """
        import shutil
        dest = backup_name(location)
        shutil.copyfile(location, dest)

    def write(self, filename=None, output="dict", attribute_indent=attribute_indent):
        """
        This method writes the dict into various outout formats. This includes a dict,
        json, and yaml

        :param filename: the file in which the dict is written
        :param output: is a string that is either "dict", "json", "yaml"
        :param attribute_indent: character indentation of nested attributes in
        """
        if filename is not None:
            location = path_expand(filename)
        else:
            location = self['meta']['location']

            # with open('data.yml', 'w') as outfile:
            #    outfile.write( yaml.dump(data, default_flow_style=True) )

        # Make a backup
        self.make_a_copy(location)

        f = os.open(location, os.O_CREAT | os.O_TRUNC |
                    os.O_WRONLY, stat.S_IRUSR | stat.S_IWUSR)
        if output == "json":
            os.write(f, self.json())
        elif output in ['yml', 'yaml']:
            # d = dict(self)
            # os.write(f, yaml.dump(d, default_flow_style=False))
            os.write(f, ordered_dump(OrderedDict(self),
                                     Dumper=yaml.SafeDumper,
                                     default_flow_style=False,
                                     indent=attribute_indent))
        elif output == "print":
            os.write(f, custom_print(self, attribute_indent))
        else:
            os.write(f, self.dump())
        os.close(f)

    def error_keys_not_found(self, keys):
        """
        Check if the requested keys are found in the dict.

        :param keys: keys to be looked for
        """
        try:
            log.error("Filename: {0}".format(self['meta']['location']))
        except:
            log.error("Filename: {0}".format(self['location']))
        log.error("Key '{0}' does not exist".format('.'.join(keys)))
        indent = ""
        last_index = len(keys) - 1
        for i, k in enumerate(keys):
            if i == last_index:
                log.error(indent + k + ": <- this value is missing")
            else:
                log.error(indent + k + ":")
            indent += "    "

    def __str__(self):
        """
        returns the json output of the dict.
        """
        return self.json()

    def json(self):
        """
        returns the json output of the dict.
        """
        return json.dumps(self, indent=attribute_indent)

    def yaml(self):
        """
        returns the yaml output of the dict.
        """
        return ordered_dump(OrderedDict(self),
                            Dumper=yaml.SafeDumper,
                            default_flow_style=False)

    def dump(self):
        """
        returns the json output of the dict.
        """
        orderedPrinter = OrderedJsonEncoder()
        return orderedPrinter.encode(self)

    def pprint(self):
        """
        uses pprint to print the dict
        """
        print(custom_print(self, attribute_indent))

    """
    def __getitem__(self, *mykeys):
        try:
            item = self.get(mykeys[0])
        except:
            self._notify_of_error(mykeys)
            sys.exit()
        return item
    """

    def get(self, *keys):
        """
        returns the dict of the information as read from the yaml file. To
        access the file safely, you can use the keys in the order of the
        access.
        Example: get("provisioner","policy") will return the value of
        config["provisioner"]["policy"] from the yaml file if it does not exists
        an error will be printing that the value does not exists. Alternatively
        you can use the . notation e.g. get("provisioner.policy")
        """
        if keys is None:
            return self

        if "." in keys[0]:
            keys = keys[0].split('.')
        element = self
        for v in keys:
            try:
                element = element[v]
            except KeyError:
                self.error_keys_not_found(keys)
                # sys.exit()
        return element

    def set(self, value, *keys):
        """
        Sets the dict of the information as read from the yaml file. To access
        the file safely, you can use the keys in the order of the access.
        Example: set("{'project':{'fg82':[i0-i10]}}", "provisioner","policy")
        will set the value of config["provisioner"]["policy"] in the yaml file if
        it does not exists an error will be printing that the value does not
        exists.  Alternatively you can use the . notation e.g.
        set("{'project':{'fg82':[i0-i10]}}", "provisioner.policy")
        """
        element = self

        if keys is None:
            return self

        if '.' in keys[0]:
            keys = keys[0].split(".")

        nested_str = ''.join(["['{0}']".format(x) for x in keys])
        # Safely evaluate an expression to see if it is one of the Python
        # literal structures: strings, numbers, tuples, lists, dicts, booleans,
        # and None. Quoted string will be used if it is none of these types.
        try:
            ast.literal_eval(str(value))
            converted = str(value)
        except ValueError:
            converted = "'" + str(value) + "'"
        exec("self" + nested_str + "=" + converted)
        return element

    def _update(self, keys, value):
        """Updates the selected key with the value

        Args:
            keys (str): key names e.g. cloudmesh.server.loglevel
            value (str): value to set
        """
        return self.set(value, keys)

    def attribute(self, keys):
        """
        TODO: document this method

        :param keys:
        """
        if self['meta']['prefix'] is None:
            k = keys
        else:
            k = self['meta']['prefix'] + "." + keys
        return self.get(k)


if __name__ == "__main__":
    config = ConfigDict({"a": "1", "b": {"c": 3}},
                        prefix="cloudmesh.debug",
                        filename="./etc/cloudmesh_debug.yaml")

    print("PPRINT")
    print(70 * "=")
    pprint(config)

    print("PRINT")
    print(70 * "=")
    print(config.pprint())
    print(config.json())

    print(70 * "=")
    print("A =", config["a"])
    config.write(config_file("/d.yaml"), output="dict")
    config.write(config_file("/j.yaml"), output="json")
    config.write(config_file("/y.yaml"), output="yaml")

    # this does not work
    # config.write(config_file("/print.yaml"), output="print")

    print("mongo.path GET =", config.get("cloudmesh.server.mongo.path"))
    print("mongo.path ATTRIBUTE =", config.attribute("mongo.path"))

    print("get A =", config.get("a"))

    print("wrong mongo.path ATTRIBUTE =", config.attribute("mongo.path.wrong"))
    print("wrong mongo.path GET =",
          config.get("cloudmesh.server.mongo.path.wrong"))

    # print config["dummy"]
    # config["x"] = "2"
    # print config["x"]
    # print config.x



from __future__ import print_function
from cloudmesh_client.default import Default
from cloudmesh_client.shell.command import command
from cloudmesh_client.shell.console import Console
from cloudmesh_client.cloud.limits import Limits
from cloudmesh_client.shell.command import PluginCommand, CloudPluginCommand


class LimitsCommand(PluginCommand, CloudPluginCommand):
    topics = {"limits": "cloud"}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command limits")

    # noinspection PyUnusedLocal
    @command
    def do_limits(self, args, arguments):
        """
        ::

            Usage:
                limits list [--cloud=CLOUD] [--tenant=TENANT] [--format=FORMAT]

                Current list data with limits on a selected project/tenant.
                The --tenant option can be used by admin only

            Options:
               --format=FORMAT  the output format [default: table]
               --cloud=CLOUD    the cloud name
               --tenant=TENANT  the tenant name

            Examples:
                cm limits list
                cm limits list --cloud=kilo --format=csv

        """
        if arguments["list"]:
            cloud = arguments["--cloud"] or Default.cloud

            if not cloud:
                Console.error("cloud doesn't exist")
                return ""

            output_format = arguments["--format"]
            tenant = arguments["--tenant"]
            result = Limits.list(cloud,
                                 output=output_format,
                                 tenant=tenant)
            Console.msg(result)
            return ""




import json
import subprocess


class Who(object):
    @classmethod
    def _run_command(cls, command):
        p = subprocess.Popen(command,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.STDOUT)
        return p.stdout.readlines()

    def __init__(self, hostname):
        self.get(hostname)
        self.parse()
        self.data = None
        self.result = None
        self.cache = {}

    def get(self, hostname):
        self.result = \
            self._run_command(["ssh", hostname, "who"])
        return self.result

    def parse(self):
        if self.result is None:
            self.data = None
        else:
            self.data = []
            for line in self.result:
                host = line.split("(")[1][:-1].replace(")", "")
                name = line.split()[0]
                self.data.append({
                    'name': name,
                    'host': host,
                })
        return self.data

    def __str__(self):
        return json.dumps(self.data, indent=4)

    def sanitize_whois(self, lines):
        content = "".join(lines)

        if "NOTICE:" in content:
            return None
        else:
            result = {}
            data = [l for l in lines if ":" in l]
            for element in data:
                if "Access to Public Interest Registry" not in element:
                    attribute, value = element.split(":", 1)
                # noinspection PyUnboundLocalVariable,PyUnboundLocalVariable
                result[attribute] = value.strip("\n")
            return result

    def whois(self):
        for entry in self.data:
            print("EEE", entry)
            host = entry["host"]
            if ".edu" in host or ".org" in host:
                host = host.split(".")
                host = ".".join(host[-2:])
                print(host)
            entry["domain"] = host

            if host in self.cache:
                data = self.cache[host]
            else:
                data = self._run_command(["whois", host])
                self.cache[host] = data

            entry["whois"] = self.sanitize_whois(data)


def main():
    host = "juliet"
    who = Who(host)
    who.whois()

    from pprint import pprint
    pprint(who.data)


if __name__ == "__main__":
    main()

"""
print

hosts = []

hosts = list(set(hosts))

print hosts

for host in hosts:
    r = whois([host]).split("\n")
    location = {}
    for line in r:
        for match in ["City:", "State:", "Country"]:
            if match in line:
                location[match] = line
    print location
"""



from __future__ import print_function

import os
import platform
from cloudmesh_client.shell.console import Console
from cloudmesh_client.common.ConfigDict import Config
from cloudmesh_client.common.ConfigDict import ConfigDict
from cloudmesh_client.common.Shell import Shell


class Sync(object):
    @classmethod
    def sync(cls, cloudname, localdir, remotedir, operation=None):
        """
        Syncs a local directory with a remote directory.
        Either from local to remote OR vice-versa
        :param cloudname:
        :param localdir:
        :param remotedir:
        :param operation: get/put
        :return:
        """
        # Get the operating system
        os_type = cls.operating_system()

        # fix the local dir path
        localdirpath = Config.path_expand(localdir)

        # check if local directory exists
        if not os.path.exists(localdirpath):
            if operation == "put":
                Console.error("The local directory [{}] does not exist."
                              .format(localdirpath))
                return None
            elif operation == "get":
                # for receiving, create local dir
                os.mkdir(localdirpath)
                Console.msg("Created local directory [{}] for sync."
                            .format(localdirpath))

        """
            rsync now works on windows machines as well.
            we install rsync (v5.4.1.20150827) on windows via chocolatey
            $ choco install rsync
        """

        host = cls.get_host(cloudname)
        if host is None:
            Console.error("Cloud [{}] not found in cloudmesh.yaml file."
                          .format(cloudname))
            return None
        else:
            args = None
            if operation == "put":
                args = [
                    "-r",
                    localdir,
                    host + ":" + remotedir
                ]
            elif operation == "get":
                args = [
                    "-r",
                    host + ":" + remotedir,
                    localdir
                ]
            # call rsync
            return Shell.rsync(*args)

    @classmethod
    def operating_system(cls):
        return platform.system().lower()

    @classmethod
    def get_host(cls, cloudname):
        """
        Method to get host for cloud
        from the cloudmesh.yaml file
        :param cloudname:
        :return:
        """
        # Get the remote host details
        # search based on cloudname
        config = ConfigDict("cloudmesh.yaml")
        clouds = config["cloudmesh"]["clouds"]

        # check if cloud exists in yaml
        if clouds[cloudname] is not None:
            # get the host for cloud
            hostname = clouds[cloudname]["cm_host"]
            return hostname
        else:
            return None

    @classmethod
    def get_hostname(cls, host):
        """
        Method to return hostname
        for a host in ssh config
        :param host:
        :return:
        """
        filename = Config.path_expand("~/.ssh/config")
        with open(filename, 'r') as f:
            lines = f.read().split("\n")

        found = False
        for line in lines:
            # search for host
            if "Host " in line:
                _host = line.strip().replace("Host ", "", 1).replace(" ", "")
                # if host found in ssh config
                if _host == host:
                    found = True
                    pass

            # search for hostname
            if "Hostname " in line and found is True:
                # return corresponding hostname
                hostname = line.strip().replace("Hostname ", "", 1).replace(" ", "")
                return hostname

    @classmethod
    def get_hostuser(cls, host):
        """
        Method to return user login
        for a host in ssh config
        :param host:
        :return:
        """
        filename = Config.path_expand("~/.ssh/config")
        with open(filename, 'r') as f:
            lines = f.read().split("\n")

        found = False
        for line in lines:
            # search for host
            if "Host " in line:
                _host = line.strip().replace("Host ", "", 1).replace(" ", "")
                # if host found in ssh config
                if _host == host:
                    found = True
                    pass

            # search for user
            if "User " in line and found is True:
                # return corresponding user
                username = line.strip().replace("User ", "", 1).replace(" ", "")
                return username



from __future__ import print_function

from cloudmesh_client.shell.console import Console
from cloudmesh_client.common.Printer import Printer
from cloudmesh_client.db import CloudmeshDatabase
from cloudmesh_client.cloud.iaas.CloudProvider import CloudProvider
from cloudmesh_client.cloud.ListResource import ListResource


class Workflow(ListResource):
    cm = CloudmeshDatabase()

    @classmethod
    def refresh(cls, cloud):
        """
        This method would refresh the workflow list by first clearing
        the database, then inserting new data
        :param cloud: the cloud name
        """

        Console.TODO("this method is not yet implemented. dont implement this yet")
        return


        return cls.cm.refresh('workflow', cloud)

    @classmethod
    def delete(cls,cloud, id):
        print (id)
        cls.cm.delete(kind="workflow", category='general' ,cm_id =id)
        return True

    @classmethod
    def list(cls, name, live=False, format="table"):
        """
        This method lists all workflows of the cloud
        :param cloud: the cloud name
        """

        # Console.TODO("this method is not yet implemented")
        # return

        try:

            elements = cls.cm.find(kind="workflow", category='general')

            # pprint(elements)

            # (order, header) = CloudProvider(cloud).get_attributes("workflow")
            order = None
            header= None
            # Console.msg(elements)
            return Printer.write(elements,
                                 order=order,
                                 header=header,
                                 output=format)
        except Exception as ex:
            Console.error(ex.message)

    @classmethod
    def details(cls, cloud, id, live=False, format="table"):
        elements = cls.cm.find(kind="workflow", category='general' ,cm_id =id)
        Console.msg(elements)
        order = None
        header= None
        # Console.TODO("this method is not yet implemented")
        return Printer.write(elements,
                                 order=order,
                                 header=header,
                                 output=format)

    @classmethod
    def save(cls, cloud, name, str):
        workflow = {
                "category": "general",
                "kind": "workflow",
                "name": name,
                "workflow_str": str
        }

        cls.cm.add(workflow, replace=False)
        cls.cm.save()

        return "Workflow saved in database!"

    @classmethod
    def run(cls,cloud,id):
        elements = cls.cm.find(kind="workflow", category='general', cm_id = id)
        Console.msg(elements)
        order = None
        Console.msg("Executing")
        header= None
        return elements
        # Console.TODO("this method is not yet implemented")
        # return Printer.write(elements,
        #                          order=order,
        #                          header=header,
        #                          output=format)



from __future__ import print_function
import os

from cloudmesh_client.common.Shell import Shell
from cloudmesh_client.logger import LOGGER
from cloudmesh_client.common.Printer import Printer
from cloudmesh_client.cloud.nova import Nova
from cloudmesh_client.cloud.group import Group
from cloudmesh_client.shell.command import command
from cloudmesh_client.shell.console import Console
from cloudmesh_client.default import Default
from cloudmesh_client.cloud.iaas.provider.openstack.CloudProviderOpenstackAPI import \
    set_os_environ
from cloudmesh_client.shell.command import PluginCommand, CloudPluginCommand

log = LOGGER(__file__)


# noinspection PyBroadException
class NovaCommand(PluginCommand, CloudPluginCommand):
    topics = {"nova": "cloud"}

    def __init__(self, context):
        self.context = context
        if self.context.debug:
            print("init command nova")

    # noinspection PyUnusedLocal
    @command
    def do_nova(self, args, arguments):
        """
        ::
        
            Usage:
                nova set CLOUD
                nova info [CLOUD] [--password]
                nova help
                nova [--group=GROUP] ARGUMENTS...

            A simple wrapper for the openstack nova command

            Arguments:
                GROUP           The group to add vms to
                ARGUMENTS       The arguments passed to nova
                help            Prints the nova manual
                set             reads the information from the current cloud
                                and updates the environment variables if
                                the cloud is an openstack cloud
                info            the environment values for OS

            Options:
                --group=GROUP   Add VM to GROUP group
                --password      Prints the password
                -v              verbose mode

        """
        # pprint(arguments)
        cloud = arguments['CLOUD'] or Default.cloud
        if not cloud:
            Console.error("Default cloud not set!")
            return ""

        group = arguments["--group"] or Default.group

        if not group:
            Console.error("Default group not set!")
            return ""

        if arguments["help"]:
            os.system("nova help")
            return ""
        elif arguments["info"]:
            set_os_environ(cloud)
            d = {}
            #
            # TODO: this naturally does not work as clouds will have
            # different parameters. ALos it does not unset previous
            # parameters from other clouds. See register
            #
            for attribute in ['OS_USERNAME',
                              'OS_TENANT_NAME',
                              'OS_AUTH_URL',
                              'OS_CACERT',
                              'OS_PASSWORD',
                              'OS_REGION']:
                try:
                    d[attribute] = os.environ[attribute]
                except:
                    Console.warning("OS environment variable {:} not found"
                                    .format(attribute))
                    d[attribute] = None
                if not arguments["--password"]:
                    d['OS_PASSWORD'] = "********"
            print(Printer.row_table(d, order=None, labels=["Variable", "Value"]))
            msg = "info. OK."
            Console.ok(msg)
            return ""
        elif arguments["set"]:
            if cloud:

                set_os_environ(cloud)

                msg = "{0} is set".format(cloud)
                Console.ok(msg)
            else:
                Console.error("CLOUD is required")

        else:  # nova ARGUMENTS...
            print("Cloud = {0}".format(cloud))
            try:
                set_os_environ(cloud)
                args = arguments["ARGUMENTS"]

                # arguments may contain multiple optional arguments
                if len(args) == 1:
                    args = args[0].split()

                result = Shell.execute("nova", args)


                print (result)
                # print(Nova.remove_subjectAltName_warning(result))

                """
                If request for nova boot,
                add the vm to group specified,
                or else add to default group
                """
                if "boot" in args:
                    # Logic to find ID of VM in the result
                    fields = []
                    for field in result.split("|"):
                        fields.append(field.strip())
                    index = fields.index('id') + 1
                    vm_id = fields[index]

                    # Add to group
                    Group.add(name=group, species="vm", member=vm_id, category=cloud)
            except Exception as ex:
                Console.error("Error executing Nova command: {}".format(ex))
            return ""



""" run with

python setup.py install; nosetests -v --nocapture  tests/cm_libcloud/test_libcloud_api.py:Test_image.test_001

nosetests -v --nocapture tests/libcloud/test_libcloud_api.py

or

nosetests -v tests/test_image.py

"""

from __future__ import print_function

from pprint import pprint
from time import sleep

from libcloud.compute.providers import get_driver
from libcloud.compute.types import Provider

from cloudmesh_client.common.ConfigDict import ConfigDict


# noinspection PyPep8Naming
class Test_libcloud_aws:
    """
        This class tests the lib cloud connection to aws
    """

    def test_001(self):
        self.conf = ConfigDict("cloudmesh.yaml")
        self.credentials = self.conf['cloudmesh']['clouds']['aws']['credentials']
        self.default = self.conf['cloudmesh']['clouds']['aws']['default']
        pprint(self.credentials)

        # TODO: use EC2 instead of AWS, use chameleon

        self.cls = cls = get_driver(Provider.EC2_US_EAST)
        self.driver = cls(
            self.credentials['EC2_ACCESS_KEY'],
            self.credentials['EC2_SECRET_KEY'])
        assert True

    def test_002(self):
        """list VMs"""
        self.nodes = self.driver.list_nodes()
        print(self.nodes)
        assert True

    def test_003(self):
        """list images"""
        self.images = self.driver.list()
        # print images[0]
        assert True

    def test__004(self):
        """list flavors"""
        self.sizes = self.driver.list_sizes()
        # print sizes
        assert True

    def test_005(self):
        # specify flavor and image
        self.myflavor = self.default['flavor']
        self.myimage = self.default['image']
        assert True

    def test_006(self):
        # Changed "name" -> "id" (diff from openstack)
        size = [s for s in self.sizes if s.id == self.myflavor][0]
        image = [i for i in self.images if i.id == self.myimage][0]
        assert True

    def test_007(self):
        """launch a new VM"""
        name = "{:}-libcloud".format(self.credentials['userid'])
        # TODO: BUG: self.image self.size, not set
        self.node = self.driver.create_node(name=name,
                                       image=self.myimage,
                                       size=self.myflavor)
        assert True

    def test_008(self):
        """check if the new VM is in the list"""
        nodes = self.driver.list_nodes()
        print(nodes)
        assert True

    def test_009(self):
        """public ip"""

        # wait the node to be ready before assigning public IP
        sleep(10)

        # public IPs
        # get the first pool - public by default
        # create an ip in the pool
        elastic_ip = self.driver.ex_allocate_address()

        # attach the ip to the node
        #
        # TODO: BUG: self.node not set
        self.driver.ex_associate_address_with_node(self.node, elastic_ip)

        # check updated VMs list to see if public ip is assigned
        nodes = self.driver.list_nodes()
        print(nodes)
        assert True

    def test_010(self):
        """remove node"""

        # delete the ip

        # delete vm
        # TODO: self.node not set
        self.node.destroy()
        assert True
